﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using SocketGlobal;
using SocketGlobal.SendData;

namespace SocketAsync_Client
{
	public partial class frmClient : Form
	{
		/// <summary>
		/// 명령어 클래스
		/// </summary>
		private CCommand m_insCommand = new CCommand();

		/// <summary>
		/// 내 소켓
		/// </summary>
		//private Socket m_socketMe = null;
		private CSocketClient m_SocketCient;

		/// <summary>
		/// 나의 상태
		/// </summary>
		enum typeState
		{
			/// <summary>
			/// 없음
			/// </summary>
			None = 0,
			/// <summary>
			/// 연결중
			/// </summary>
			Connecting,
			/// <summary>
			/// 연결 완료
			/// </summary>
			Connect,
		}

		/// <summary>
		/// 나의 상태
		/// </summary>
		private typeState m_typeState = typeState.None;

		public frmClient()
		{
			InitializeComponent();

			//유아이 기본으로 세팅
			UI_Setting(typeState.None);

		}

		private void btnSend_Click(object sender, EventArgs e)
		{
			switch (m_typeState)
			{
				case typeState.None:	//기본
					if ("" == txtMsg.Text)
					{
						//입력값이 없으면 리턴
						MessageBox.Show("아이디를 넣고 시도해 주세요");
						return;
					}
					else
					{
						//아이디가 있으면 로그인 시작

						//유아이를 세팅하고
						UI_Setting(typeState.Connecting);

						//소켓 생성
						Socket socketServer = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
						IPEndPoint ipepServer = new IPEndPoint(IPAddress.Parse("127.0.0.1"), Convert.ToInt32(txtPort.Text));
						//IPEndPoint ipepServer = new IPEndPoint(IPAddress.Parse("192.168.194.4"), Convert.ToInt32(txtPort.Text));

						SocketAsyncEventArgs saeaServer = new SocketAsyncEventArgs();
						saeaServer.RemoteEndPoint = ipepServer;
						//연결 완료 이벤트 연결
						saeaServer.Completed += new EventHandler<SocketAsyncEventArgs>(Connect_Completed);

						//서버 메시지 대기
						socketServer.ConnectAsync(saeaServer);
					}
					break;

				case typeState.Connect:	//접속 상태
					//이상태에서는 메시지를 보낸다.
					SendMsg(CCommand.Command.Msg, txtMsg.Text);
					txtMsg.Text = "";
					break;
			}
		}

		/// <summary>
		/// 연결 완료
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Connect_Completed(object sender, SocketAsyncEventArgs e)
		{
			this.m_SocketCient = new CSocketClient((Socket)sender);
			this.m_SocketCient.OnConnectComplete += M_SocketCient_OnConnectComplete;
			this.m_SocketCient.OnMessageComplete += M_SocketCient_OnMessageComplete;
			this.m_SocketCient.OnDisconnect += M_SocketCient_OnDisconnect;
			this.m_SocketCient.OnSendComplete += M_SocketCient_OnSendComplete;

			this.m_SocketCient.BeginReceive();
        }

		

		private void M_SocketCient_OnConnectComplete()
		{
			//DisplayMsg("*** 서버 연결 성공 ***");
			//서버 연결이 성공하면 id체크를 시작한다.
			this.Login();
		}

		private void M_SocketCient_OnMessageComplete(CSendData sdMessage)
		{
			//명령어를 분리한다.
			//mdRecieveMsg.CutCommand();


			switch (sdMessage.Command)
			{
				case CCommand.Command.None: //없다
					break;
				case CCommand.Command.Msg:    //메시지인 경우
					this.Command_Msg(sdMessage.Data_Byte_ToString());
					break;
				case CCommand.Command.ID_Check_Ok:  //아이디 성공
					this.SendMeg_IDCheck_Ok();
					break;
				case CCommand.Command.ID_Check_Fail:    //아이디 실패
					this.SendMeg_IDCheck_Fail();
					break;
				case CCommand.Command.User_Connect: //다른 유저가 접속 했다.
					this.SendMeg_User_Connect(sdMessage.Data_Byte_ToString());
					break;
				case CCommand.Command.User_Disonnect:   //다른 유저가 접속을 끊었다.
					this.SendMeg_User_Disconnect(sdMessage.Data_Byte_ToString());
					break;
				case CCommand.Command.User_List:    //유저 리스트 갱신
					this.SendMeg_User_List(sdMessage.Data_Byte_ToString());
					break;

				case CCommand.Command.Image:
					this.SendMeg_User_Image(sdMessage.Data_Byte);
                    break;
			}
		}

		private void M_SocketCient_OnDisconnect()
		{
			//유아이를 세팅하고
			UI_Setting(typeState.None);

			DisplayMsg("*** 서버 연결 끊김 ***");
		}

		private void M_SocketCient_OnSendComplete()
		{
			
		}

		/// <summary>
		/// UI 세팅
		/// </summary>
		/// <param name="typeSet"></param>
		private void UI_Setting(typeState typeSet)
		{
			//들어온 값을 세팅하고
			m_typeState = typeSet;

			switch (typeSet)
			{
				case typeState.None:	//기본
					if (false == InvokeRequired)
					{
						btnSend.Text = "로그인";
					}
					else
					{
						this.Invoke(new Action(
						delegate()
						{
							btnSend.Text = "로그인";
						}));
					}
					break;
				case typeState.Connecting:	//연결중
					txtMsg.Enabled = false;
					btnSend.Text = "연결중";
					btnSend.Enabled = false;
					break;
				case typeState.Connect:	//연결완료
					this.Invoke(new Action(
						delegate()
						{
							txtMsg.Enabled = true;
							btnSend.Text = "보내기";
							btnSend.Enabled = true;
						}));
					break;
			}
		}

		/// <summary>
		/// 메시지 출력
		/// </summary>
		/// <param name="sMsg"></param>
		private void Command_Msg(string sMsg)
		{
			DisplayMsg(sMsg);
		}

		/// <summary>
		/// 아이디 성공
		/// </summary>
		private void SendMeg_IDCheck_Ok()
		{
			this.Invoke(new Action(
						delegate()
						{
							labID.Text = txtMsg.Text;
							txtMsg.Text = "";
						}));

			//UI갱신
			UI_Setting(typeState.Connect);

			//아이디확인이 되었으면 서버에 로그인 요청을 하여 로그인을 끝낸다.
			SendMsg(CCommand.Command.Login, "");
		}

		/// <summary>
		/// 아이디체크 실패
		/// </summary>
		private void SendMeg_IDCheck_Fail()
		{
			DisplayMsg("로그인 실패 : 다른 아이디를 이용해 주세요.");
			//연결 끊기
			this.m_SocketCient.Disconnect_Call();
		}

		/// <summary>
		/// 접속한 유저가 있다.
		/// </summary>
		private void SendMeg_User_Connect(string sUserID)
		{
			this.Invoke(new Action(
						delegate()
						{
							listUser.Items.Add(sUserID);
						}));
		}

		/// <summary>
		/// 접속을 끊은 유저가 있다.
		/// </summary>
		/// <param name="sUserID"></param>
		private void SendMeg_User_Disconnect(string sUserID)
		{
			this.Invoke(new Action(
						delegate()
						{
							listUser.Items.RemoveAt(listUser.FindString(sUserID));
						}));
		}

		/// <summary>
		/// 유저리스트 
		/// </summary>
		/// <param name="sUserList"></param>
		private void SendMeg_User_List(string sUserList)
		{
			this.Invoke(new Action(
				delegate()
				{
					//리스트를 비우고
					listUser.Items.Clear();

					//리스트를 다시 채워준다.
					string[] sList = sUserList.Split(',');
					for (int i = 0; i < sList.Length; ++i)
					{
						listUser.Items.Add(sList[i]);
					}

				}));
		}

		private void SendMeg_User_Image(byte[] byteData)
		{
			MemoryStream mStream = new MemoryStream();
			Image image = Image.FromStream(new MemoryStream(byteData));
			/*
			byte[] pData = byteData;
			mStream.Write(pData, 0, pData.Length);
			Bitmap bm = new Bitmap(mStream, false);
			mStream.Dispose();*/

			this.Invoke(new Action(
						delegate ()
						{
							pbDownImage.Image = image;
						}));
		}

		/// <summary>
		/// 받아온 메시지를 출력 한다.
		/// </summary>
		/// <param name="nMessage"></param>
		/// <param name="nType"></param>
		private void DisplayMsg(String nMessage)
		{
			StringBuilder buffer = new StringBuilder();

			//출력할 메시지 완성
			buffer.Append(nMessage);

			//출력
			this.Invoke(new Action(
						delegate()
						{
							listMsg.Items.Add(nMessage);
						}));

		}



		/// <summary>
		/// 아이디 체크 요청
		/// </summary>
		private void Login()
		{
			SendMsg(CCommand.Command.ID_Check, txtMsg.Text);
		}

		private void Logout()
		{
			SendMsg(CCommand.Command.Logout, "");
		}

		public void SendMsg(CCommand.Command typeCommand, byte[] byteData)
		{
			CSendData dsSendMsg = new CSendData();

			//데이터를 넣고
			dsSendMsg.Command = typeCommand;
			dsSendMsg.Data_Byte = byteData;

			this.m_SocketCient.SendMsg(dsSendMsg);
		}

		/// <summary>
		/// 서버로 메시지를 전달 합니다.
		/// </summary>
		/// <param name="sMsg"></param>
		private void SendMsg(CCommand.Command typeCommand, string sMsg)
		{
			CSendData dsSendMsg = new CSendData();

			//데이터를 넣고
			dsSendMsg.Command = typeCommand;
			dsSendMsg.Data_String = sMsg;

			this.m_SocketCient.SendMsg(dsSendMsg);
		}

		
		/// <summary>
		/// 테스트용 자동 메시지 타이머
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void timer1_Tick(object sender, EventArgs e)
		{
			SendMsg(CCommand.Command.Msg, txtAutoMsg.Text);
		}

		private void btnAutoMsg_Click(object sender, EventArgs e)
		{
			if (true == timer1.Enabled)
			{
				timer1.Enabled = false;
			}
			else
			{
				timer1.Enabled = true;
			}
		}

		private void btnImageSend_Click(object sender, EventArgs e)
		{
			//이미지 전송

			//명령어 만들기
			byte[] byteTemp = File.ReadAllBytes(txtDir.Text);
			//파일 읽기
			/*byte[] buff = File.ReadAllBytes(txtDir.Text);
			FileStream fs = new FileStream(txtDir.Text, FileMode.Open);
			byteTemp = new byte[fs.Length];
			fs.Read(byteTemp, 0, byteTemp.Length);
			fs.Close();*/


			//전송 요청
			this.SendMsg(CCommand.Command.Image, byteTemp);
		}



	}
}
